import javax.microedition.rms.RecordComparator;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordFilter;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotOpenException;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
   This class implements a simple address book for
   demonstration purposes.  It stores records consisting
   of a String name and a String phone number field.  This
   class defines two inner classes, one record comparator
   and one record filter for use in retrieving records.
*/
public class AddressBook 
{
  private static final String RECORD_STORE_NAME = 
    "address-book";

  private RecordStore recordStore;

  public AddressBook() throws RecordStoreException
  {
    super();
    recordStore = 
      RecordStore.openRecordStore(RECORD_STORE_NAME, 
				  true);
  }

  void close() throws RecordStoreException
  {
    try
      {
	recordStore.closeRecordStore();
      }
    catch (RecordStoreNotOpenException rsno)
      {

      }
  }

  /**
     Gets the record store used by this object.

     @return a reference to the RecordStore used by this
     object.
  */
  public RecordStore getRecordStore()
  {
    return recordStore;
  }

  /**
     Adds the specified record to this address book's
     record store.

     @param name the name of the entry being added.
     @param phone the phone number for the entry being
     added.

     @throws RecordStoreException if there is a problem 
     adding the record. 
  */
  public void addRecord(String name, String phone) 
    throws RecordStoreException
  {
    ByteArrayOutputStream baos = 
      new ByteArrayOutputStream();
    DataOutputStream dos = new DataOutputStream(baos);

    try
      {
	dos.writeUTF(name);
	dos.writeUTF(phone);
      }
    catch (IOException ioe)
      {
	ioe.printStackTrace();
      }
    int id = 
      recordStore.addRecord(baos.toByteArray(),
			    0, 
			    baos.toByteArray().length);
    System.out.println("Record id = " + id);
  }

  /**
     A RecordEnumerator that orders records by
     lexicographic ordering of the name field of the
     record.
  */
  RecordEnumeration getMatchesByName(String matchKey) 
    throws RecordStoreNotOpenException
  {
    MatchAllNamesFilter filter =
      new MatchAllNamesFilter(matchKey);

    AlphabeticalOrdering comparator = 
      new AlphabeticalOrdering();
    return recordStore.enumerateRecords(filter, 
					comparator,
					false);
  }

  /**
     A RecordFilter that identifies a match if the
     candidate name (the first field in the candidate
     record) 1) exactly matches the name of an enumeration
     element, or 2) if the enumeration element name string
     begins with the candidate name.  Returns true if
     there is a match, false otherwise.
  */
  class MatchAllNamesFilter implements RecordFilter
  {
    String requestString;

    public MatchAllNamesFilter(String matchKey)
    {
      requestString = matchKey;
    }

    public boolean matches(byte [] candidate)
    {
      ByteArrayInputStream bais = 
	new ByteArrayInputStream(candidate);
      DataInputStream dis = new DataInputStream(bais);
      String name = null;
    
      try
	{
	  name = dis.readUTF();
	  if (name.indexOf(requestString) == 0)
	    return true;
	  else
	    return false;
	}
      catch (IOException ioe)
	{
	  ioe.printStackTrace();
	}
      return true;
    }
  }

  /**
     This inner class implements a RecordComparator whose
     policy it is to do alphabetic ordering.
  */
  class AlphabeticalOrdering implements RecordComparator
  {
    /**
       No-arg constructor.
    */
    public AlphabeticalOrdering()
    {

    }

    public int compare(byte [] rec1, byte [] rec2)
    {
      ByteArrayInputStream bais1 =
	new ByteArrayInputStream(rec1);
      DataInputStream dis1 = new DataInputStream(bais1);

      ByteArrayInputStream bais2 =
	new ByteArrayInputStream(rec2);
      DataInputStream dis2 = new DataInputStream(bais2);

      String name1 = null;
      String name2 = null;
      try
	{
	  name1 = dis1.readUTF();
	  name2 = dis2.readUTF();
	}
      catch (IOException ioe)
	{
	  ioe.printStackTrace();
	}

      if (name1 == null || name2 == null)
	return 0;

      int result = name1.compareTo(name2);
      if (result < 0)
	return RecordComparator.PRECEDES;
      else if (result == 0)
	return RecordComparator.EQUIVALENT;
      else 
	return RecordComparator.FOLLOWS;
    }
  }

  /**
     Deletes all records in the data store.  With current
     implementations, a faster way to delete all records
     may be to remove the data store and recreate it,
     rather than deleting each record one at a time!
  */
  void deleteAllRecords()
  {
    try
      {
	RecordEnumeration re = 
	  recordStore.enumerateRecords(null, null, false);
	while (re.hasNextElement())
	  {
	    int id = re.nextRecordId();
	    recordStore.deleteRecord(id);
	  }
      }
    catch (RecordStoreException rse)
      {
	rse.printStackTrace();
      }
  }

  /**
     Gets the statistics of the record store used by this
     address book.

     @return a String of statistics data. 
  */
  public String getStatistics()
  {
    int numRecords = 0;
    int space = 0;
    StringBuffer stats = new StringBuffer("Records: ");

    try
      {
	numRecords = recordStore.getNumRecords();
	space = recordStore.getSizeAvailable();
      }
    catch (RecordStoreException rse)
      {
	rse.printStackTrace();
      }
    stats.append(String.valueOf(numRecords));
    stats.append("\n\n");
    stats.append("Available bytes: ");
    stats.append(String.valueOf(space));

    return stats.toString();
  }
}

