import java.io.InputStream;
import java.io.IOException;

import javax.microedition.io.Connection;
import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;

import javax.microedition.lcdui.Displayable;

/**
   This class defines the helper object used by the
   ResourceDisplay class. It makes the actual connection
   to the HTTP resource, sends the request, and fetches
   the response. It places the response meta-information
   in a buffer. This class provides a method that enables
   another object to get this information as a String
   object asynchronously. 

   This class also writes diagnostic output to standard
   output for purposes of demonstration. The output will
   appear in the J2MEWTK emulator window.

   Note that this class implements Runnable. It can be
   used by a program to do its work asynchronously,
   controlled by a thread different from the application
   main thread. In this connection demo, a separate thread
   is not spawned to control this instance, because a
   separate thread already controls the ResourceDisplay
   instance, which uses this instance.
*/
public class HttpResource implements Runnable
{
  private static Displayable instance; 

  // The URI representing the resource to fetch.
  private String uri;

  // A buffer to hold the resource information.
  private StringBuffer contents = new StringBuffer();

  // The connection to the resource.
  private Connection conn;

  // The reference to the HTTP connection.
  private HttpConnection httpConn;

  // The connection's input stream.
  private InputStream is;

  // The value of the HTTP status attribute field.
  private int status = -1;

  /**
     Constructor.

     @param uri the URI that specifies the resource to
     fetch.
  */
  public HttpResource(String uri)
  {
    super();
    this.uri = uri;
  }

  private String userAgentID()
  {
    StringBuffer buf = new StringBuffer();

    String config = 
      System.getProperty("microedition.configuration");
    String profile = 
      System.getProperty("microedition.profiles");

    buf.append("Configuration/");
    buf.append(config);
    buf.append(" Profile/");
    buf.append(profile);

    return buf.toString();
  }

  /**
     Runs this object.  Connects to the URI, sends the
     request, receives the response, and parses the
     response message.
  */
  public void run()
  {
    System.out.println("Connection class name = " + 
		       conn.getClass().getName());
    connect();
    parse();
    System.out.println(getResourceMetaInfo());
    try
      {
	conn.close();
      }
    catch (IOException ioe)
      {
	System.out.println(ioe.getMessage());
	ioe.printStackTrace();
      }
  }

  /**
     Connects to the origin server which hosts the URI.
     If an exception occurs during connection, this method
     catches it and gives no indication of the error
     except to write diagnostics to standard output.
  */
  protected void connect()
  {
    try
      {
	while (true)
	  {
	    // Connection is in the "setup" state.
	    conn = Connector.open(uri);
	    httpConn = (HttpConnection) conn;

	    httpConn.setRequestProperty("method", 
					HttpConnection.HEAD);
	    httpConn.setRequestProperty("User-Agent", 
					userAgentID());

	    // Connection is in the "connected" state.
	    if (resourceRelocated())
	      {
		uri = httpConn.getHeaderField("location");

		// Connection in "closed" state after the 
		// call to close().
		conn.close();
	      }
	    else
	      {
		break;
	      }
	  }
	if (serverError())
	  {
	    conn.close();
	    return;
	  }
	// Connection is in the "connected" state.
	is = httpConn.openInputStream();
	System.out.println("Input stream class name = " + 
			   is.getClass().getName());

	int responseCode = httpConn.getResponseCode();
	printResponseCode(responseCode);
      }
    catch (IOException ioe)
      {
	contents.append(ioe.getMessage());
	System.out.println(ioe.getMessage());
	ioe.printStackTrace();
      }
  }

  private boolean resourceRelocated()
  {
    boolean relocated = false;
    try
      {
	status = httpConn.getResponseCode();
	if (status == HttpConnection.HTTP_MOVED_TEMP ||
	    status == HttpConnection.HTTP_MOVED_PERM ||
	    status == HttpConnection.HTTP_TEMP_REDIRECT)
	  {
	    relocated = true;
	  }
      }
    catch (IOException ioe)
      {
	System.out.println(ioe.getMessage());
	ioe.printStackTrace();
      }
    return relocated;
  }

  private boolean serverError()
  {
    boolean error = false;
    try
      {
	status = httpConn.getResponseCode();
	if ((status == HttpConnection.HTTP_NOT_IMPLEMENTED) ||
	    (status == HttpConnection.HTTP_VERSION) ||
	    (status == HttpConnection.HTTP_INTERNAL_ERROR) ||
	    (status == HttpConnection.HTTP_GATEWAY_TIMEOUT) ||
	    (status == HttpConnection.HTTP_BAD_GATEWAY)) 
	  {
	    error = true;
	  }
      }
    catch (IOException ioe)
      {
	error = true;
	System.out.println(ioe.getMessage());
	ioe.printStackTrace();
      }
    return error;
  }

  private void parse()
  {
    if (httpConn == null)
      return;

    String protocol = httpConn.getProtocol();
    contents.append("Protocol: " + protocol + "\n");

    String type = httpConn.getType();
    contents.append("Type: " + type + "\n");

    String encoding = httpConn.getEncoding();
    contents.append("Encoding: " + encoding + "\n");

    long length = httpConn.getLength();
    contents.append("Length: " + length + "\n");

    String uri = httpConn.getURL();
    contents.append("URL: " + uri + "\n");

    String host = httpConn.getHost();
    contents.append("Host: " + host + "\n");

    String query = httpConn.getQuery();
    contents.append("Query: " + query + "\n");

    String requestMethod = httpConn.getRequestMethod();
    contents.append("Method: " + requestMethod + "\n");
  }

  private void printResponseCode(int code)
  {
    System.out.print("Response code : ");
    switch (code)
      {
      case HttpConnection.HTTP_ACCEPTED:
	System.out.println("HTTP_ACCEPTED");
	break;
      case HttpConnection.HTTP_BAD_GATEWAY:
	System.out.println("HTTP_BAD_GATEWAY");
	break;
      case HttpConnection.HTTP_BAD_METHOD:
	System.out.println("HTTP_BAD_METHOD");
	break;
      case HttpConnection.HTTP_BAD_REQUEST:
	System.out.println("HTTP_BAD_REQUEST");
	break;
      case HttpConnection.HTTP_CONFLICT:
	System.out.println("HTTP_CONFLICT");
	break;
      case HttpConnection.HTTP_CREATED:
	System.out.println("HTTP_CREATED");
	break;
      case HttpConnection.HTTP_FORBIDDEN:
	System.out.println("HTTP_BAD_FORBIDDEN");
	break;
      case HttpConnection.HTTP_GATEWAY_TIMEOUT:
	System.out.println("HTTP_GATEWAY_TIMEOUT");
	break;
      case HttpConnection.HTTP_GONE:
	System.out.println("HTTP_GONE");
	break;
      case HttpConnection.HTTP_NO_CONTENT:
	System.out.println("HTTP_NO_CONTENT");
	break;
      case HttpConnection.HTTP_NOT_ACCEPTABLE:
	System.out.println("HTTP_NOT_ACCEPTABLE");
	break;
      case HttpConnection.HTTP_NOT_FOUND:
	System.out.println("HTTP_NOT_FOUND");
	break;
      case HttpConnection.HTTP_OK:
	System.out.println("HTTP_OK");
	break;
      case HttpConnection.HTTP_PROXY_AUTH:
	System.out.println("HTTP_PROXY_AUTH");
	break;
      case HttpConnection.HTTP_UNAVAILABLE:
	System.out.println("HTTP_UNAVAILABLE");
	break;
      case HttpConnection.HTTP_VERSION:
	System.out.println("HTTP_VERSION");
	break;
      default:
	System.out.println();
      }
  }

  /**
     Gets the resource meta-information.  

     @returns the meta-information returned by the origin
     server in its response message.
  */
  public String getResourceMetaInfo()
  {
    return contents.toString();
  }
}
