import javax.microedition.midlet.MIDlet;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

import java.io.DataInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.EOFException;
import java.io.IOException;
import java.io.Reader;
import java.io.UTFDataFormatException;
import java.io.UnsupportedEncodingException;

import java.util.Hashtable;

/**
  The second version of the I18NDemo application.  

  <p>This version also demonstrates a naive way to
  define localized resources.  It reads a file
  that's part of the application JAR file (not the
  JAD file) to load localized resources.  The file
  consists of a set of key-value pairs, one per
  line, that represent localized strings.  The
  MIDlet must then parse the file contents and build
  an internal hash table for lookup.

  <p>This approach requires too much processing of
  the stream that contains the file contents of the
  localized resources.  Furthermore, this approach
  is not amenable to localized resources that are
  not strings.
*/
public class I18NDemo2 extends MIDlet
{
  // The file that contains the resources for the
  // active locale.
  private String resourceFile;

  // The locale specified for the execution of this
  // MIDlet.
  private String locale;

  // The default character encoding set used by the
  // platform.
  private String encoding;

  // The HashTable that contains the localized
  // resources.
  private Hashtable resources = new Hashtable();

  // The Displayable.  This component is displayed
  // on the screen.
  private HelloForm2 form;

  // The Display instance.  This object manages all
  // Displayable components for this MIDlet.
  private Display display;

  // An instance of the MIDlet.
  private static I18NDemo2 instance; 

  /**
     No-arg constructor.
  */
  public I18NDemo2()
  {
    super();
    instance = this;
  }

  /**
     Gets the instance of this class that exists in
     the running application.

     @return the instance created when the
     application starts up.
  */
  public static I18NDemo2 getInstance()
  {
    return instance;
  }

  /**
     Start the MIDlet.  Get the current locale name.
     Use it to build the name of a file that
     contains localized resources for the locale.
     The resource file is located in the application
     JAR file.
  */
  public void startApp()
  {
    System.out.println("Encoding = " + 
		       System.getProperty("microedition.encoding"));
    // Retrieve the locale from the AMS software.
    // The locale must be set before this MIDlet
    // executes.
    locale = 
      System.getProperty("microedition.locale");

    // The names of the l10n resource files follow
    // the form: <language>_<country>.txt.
    // Construct the file name string and pass it to
    // the method that opens the file and retrieves
    // the contents.
    resourceFile = locale + ".txt";
    int status = loadResources(resourceFile);

    if (status < 0)
      {
	quit();
	return;
      }

    // Create a Displayable widget.  Get the
    // localized String that represents the title of
    // the Form.
    String formTitle = getResource("title");
    form = new HelloForm2(formTitle);

    // This app simply displays the single form
    // created above.
    display = Display.getDisplay(this);
    display.setCurrent(form);
  }

  /**
     Loads the user-defined application resources
     from the specified file.  The file is part of
     the application JAR file on a real device.  The
     J2MEWTK stores the file in the application JAR
     fine in the application's bin/ directory.

     @param file the name of the user-defined
     application resource file.
  */
  private int loadResources(String file)
  {
    Class c = getClass();

    if (file == null)
      {
	return -1 ;
      }
    InputStream is = null;
    is = c.getResourceAsStream(file);
    if (is == null)
      {
	return -1;
      }
    Reader reader = new InputStreamReader(is);
    processStream(reader);
    return 0;
  }

  /**

  */
  private void processStream(Reader stream)
  {
    if (stream == null)
      {
	return;
      }
    StringBuffer key = new StringBuffer();;
    StringBuffer value = new StringBuffer();;
    while (true)
      {
	// Read a line.  It's assumed that each line
	// contains a single key and value,
	// separated by a colon.  If -1 we've
	// reached the end of the file.
	key.delete(0, key.length());
	value.delete(0, value.length());
	int status = readLine(key, value, stream);
	if (status == -1)
	  {
	    break;
	  }

	// Put this resource in the hash table of
	// application resources.
	resources.put(key, value);
      }
  }

  /**
     Reads and processes the next non-blank line
     from the stream.  The format of the line is
     expected to be <key>[ \t]*:[ \t]*<value>, where
     <key> and <value> are tokens consisting of
     alphanumeric or punctuation characters, but no
     white space orcontrol characters.
  */
  private int readLine(StringBuffer key, 
		       StringBuffer value, 
		       Reader stream)
  {
    if (key == null || value == null || 
	stream == null)
      {
	return -1;
      }

    try
      {
	char c;
	while (true)
	  {
	    // Skip new line characters. 
	    while (true)
	      {
		c = (char) stream.read();
		if (c == '\n')
		  {
		    continue;
		  }
		break;
	      }

	    if (!isWhiteSpace(c) && !isDelimeter(c))
	      {
		key.append(c);
	      }

	    // Skip any leading white space.
	    while (true)
	      {
		c = (char) stream.read();
		if (isWhiteSpace(c))
		  {
		    continue;
		  }
		break;
	      }

	    if (!isWhiteSpace(c) && !isDelimeter(c))
	      {
		key.append(c);
	      }

	    // Read key.
	    while (true)
	      {
		c = (char) stream.read();
		if (isWhiteSpace(c) || isDelimeter(c))
		  {
		    break;
		  }
		else
		  {
		    key.append(c);
		  }
	      }
	    
	    // Skip delimeter and any leading or
	    // trailing white space.
	    while (true)
	      {
		c = (char) stream.read();
		if (isWhiteSpace(c) || isDelimeter(c))
		  {
		    continue;
		  }
		value.append(c);
		break;
	      }

	    // Read the rest of the value token.  
	    while (true)
	      {
		c = (char) stream.read();
		if (c == '\n')
		  {
		    break;
		  }
		else
		  {
		    value.append(c);
		  }
	      }
	    break;
	  }
      }
    catch (IOException ioe)
      {
	ioe.printStackTrace();
	return -1;
      }
    return 0;
  }

  /**

  */
  private boolean isWhiteSpace(char c)
  {
    if (c == ' ' || c == '\t')
      {
	return true;
      }
    else
      {
	return false;
      }
  }

  /**

  */
  private boolean isDelimeter(char c)
  {
    if (c == ':')
      {
	return true;
      }
    return false;
  }

  /**
     Returns the value associated with the specified
     key from the bundle of application resources.

     @param key the key of the key-value pair.

     @returns the value associated with the
     specified key.
  */
  private String getResource(String key)
  {
    if (resources == null)
      {
	return null;
      }
    return (String) resources.get(key);
  }

  /**
     Quit execution.  Requests the implementation to
     terminate this MIDlet.
  */
  public void quit()
  {
    notifyDestroyed();
  }

  public void destroyApp(boolean destroy)
  {

  }
    
  public void pauseApp()
  {

  }
}


