import java.util.Hashtable;

/**
   This class defines the base class for defining
   localized application resources.  It implements a
   subset of the J2SE java.util.ResourceBundle class, but
   adheres to the interface defined by that class.
*/
public abstract class ResourceBundle
{
  /**
     The "parent" resources.  If a resource is not found
     in this bundle, the parent bundle is searched.
  */
  protected ResourceBundle parent;

  /**
     No-arg constructor. 
  */
  public ResourceBundle()
  {
    super();
  }

  /**
     Gets the resource bundle with the specified class
     name.  The class name already contains a language and
     country code designation in the standard format.  For
     instance, a resource bundle class name might be
     "I18NDemoResources_fr_FR".

     @param className the full class name, such as
     "I18NDemoResources_fr_FR"
     
     @returns the resource bundle object. 
  */
  public static ResourceBundle getBundle(String className)
    throws IllegalArgumentException, 
    MissingResourceException
  {
    return ResourceBundle.getBundle(className, "");
  }

  /**
     Gets the resource bundle with the specified base
     name.

     @param baseName the fully qualified class name of the
     bundle to retrieve.  For example, the base name of
     "I18NDemo_fr_FR" is "I18NDemo".

     @param the locale string that represents the locale
     for which the resource bundle should be retrieved.
     The expected form is <lang>.<country> according to
     ISO 639 and ISO 3166, respectively.

     @returns the resource bundle to return
  */
  public static ResourceBundle getBundle(String baseName, 
					 String locale)
    throws IllegalArgumentException, 
    MissingResourceException
  {
    Class c;

    if (baseName == null)
      {
	throw new 
	  IllegalArgumentException("No basename.");
      }
    String className = baseName + "_" + locale;
    ResourceBundle bundle = null;

    try
      {
	c = Class.forName(className);
	bundle = (ResourceBundle) c.newInstance();
      }
    catch (ClassNotFoundException cnfe)
      {
	throw new 
	  MissingResourceException("Class not found.");
      }
    catch (InstantiationException ie)
      {
	throw new 
	  MissingResourceException("Can't instantiate.");
      }
    catch (IllegalAccessException iae)
      {
	throw new 
	  MissingResourceException("Can't access.");
      }
    return bundle;
  }
  
  /**
     Retrieves the object with the specified key.  If the
     key is not found, the parent bundle is searched.

     @param key the object key
     @return the object with the specified key
  */
  public final Object getObject(String key) 
    throws MissingResourceException
  {
    Object obj;

    if (key == null)
      {
	throw new NullPointerException();
      }
    obj = handleGetObject(key);
    if (obj == null && parent != null)
      {
	obj = parent.getObject(key);
      }
    if (obj == null)
      {
	throw new MissingResourceException();
      }
    return obj;
  }

  /**
     Searches this resource bundle for the object with the
     specified key.

     @param key the lookup key for the desired object.
     @return the object with the specified key
  */
  protected abstract Object handleGetObject(String key);
}

