import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.io.StreamConnectionNotifier;

import java.io.IOException;

/**
   This class implements a service that listens for client
   connection requests on a well-known socket.

   It opens a connection on a predefined port number. It
   then blocks on this port, waiting for a client
   connection request.

   When a request arrives, it accepts the request, and
   opens a new socket connection. These two steps result
   in the implementation notifying the client
   implementation of the new socket connection.

   This server then spawns a component, and hands it the
   new connection object. The component runs in a separate
   thread. The component is now free to communicate with
   the client asynchronously from the server's continued
   operation.
*/
public class ServerSocket implements Runnable
{
  // The default port on which the well-known socket is
  // established.
  public static final String DEFAULT_PORT = "9876";

  // The port on which the well-known socket is
  // established.
  private String wellKnownPort; 

  // The URI this server uses to open its well-known
  // socket.
  private String uri;

  // Connection for the well-known socket.
  private StreamConnectionNotifier wellKnownConn;

  // A socket connection that connects a client.
  private StreamConnection clientConn;

  /**
     Constructor for subclasses.
  */
  protected ServerSocket()
  {
    super();
  }

  /**
     Constructor.

     @param port the well-known port on which to establish
     this object as a listener.
  */
  public ServerSocket(String port)
  {
    this();
    if (port == null)
      {
	wellKnownPort = DEFAULT_PORT;
      }
    else 
      {
	wellKnownPort = port;
      }
    setURI(port);
  }

  protected void setURI(String port)
  {
    StringBuffer buf = new StringBuffer("socket://:");
    buf.append(port);
    uri = buf.toString();
  }

  /**
     Run this server. This method should be called
     explicitly after constructing this object.  It starts
     listening for client requests on the well-known
     socket.  It is up to the caller to start this
     execution in a separate thread.
  */
  public void run()
  {
    while (true)
      {
	try
	  {
	    // Open a well-known socket connection for
	    // this "service". 
	    wellKnownConn = (StreamConnectionNotifier) 
	      Connector.open(uri);

	    // Listen for connection requests. This call
	    // blocks until a connection request is
	    // received.
	    clientConn = wellKnownConn.acceptAndOpen();

	    // Instantiate a server agent, the object that
	    // presents the service to the client. Each
	    // instance communicates with one client.
	    // Spawn a thread to communicate with the
	    // client making the connection request.
	    ServerAgent agent = new ServerAgent(clientConn);
	    Thread thread = new Thread(agent);
	  }
	catch (IOException ioe)
	  {
	    System.out.println(ioe.getMessage());
	    ioe.printStackTrace();
	    break;
	  }
      }
  }
}
